<?php
/**
 * Element for HTML_QuickForm that emulate a multi-select.
 *
 * The HTML_QuickForm_advmultiselect package adds an element to the
 * HTML_QuickForm package that is two select boxes next to each other
 * emulating a multi-select.
 *
 * PHP versions 4 and 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   HTML
 * @package    HTML_QuickForm_advmultiselect
 * @author     Laurent Laville <pear@laurent-laville.org>
 * @copyright  1997-2005 The PHP Group
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://pear.php.net/package/HTML_QuickForm_advmultiselect
 */

require_once 'HTML/QuickForm/select.php';

/**
 * Element for HTML_QuickForm that emulate a multi-select.
 *
 * The HTML_QuickForm_advmultiselect package adds an element to the
 * HTML_QuickForm package that is two select boxes next to each other
 * emulating a multi-select.
 *
 * PHP versions 4 and 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   HTML
 * @package    HTML_QuickForm_advmultiselect
 * @author     Laurent Laville <pear@laurent-laville.org>
 * @copyright  1997-2005 The PHP Group
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    Release: @package_version@
 * @link       http://pear.php.net/package/HTML_QuickForm_advmultiselect
 */
class HTML_QuickForm_advmultiselect extends HTML_QuickForm_select
{
    /**
     * Prefix function name in javascript move selections
     *
     * @var        string
     * @access     private
     */
    var $_jsPrefix;

    /**
     * Postfix function name in javascript move selections
     *
     * @var        string
     * @access     private
     */
    var $_jsPostfix;

    /**
     * Associative array of the multi select container attributes
     *
     * @var        array
     * @access     private
     */
    var $_tableAttributes;

    /**
     * Associative array of the add button attributes
     *
     * @var        array
     * @access     private
     */
    var $_addButtonAttributes;

    /**
     * Associative array of the remove button attributes
     *
     * @var        array
     * @access     private
     */
    var $_removeButtonAttributes;

    /**
     * Associative array of the unselected item box attributes
     *
     * @var        array
     * @access     private
     */
    var $_attributesUnselected;

    /**
     * Associative array of the selected item box attributes
     *
     * @var        array
     * @access     private
     */
    var $_attributesSelected;

    /**
     * Associative array of the internal hidden box attributes
     *
     * @var        array
     * @access     private
     */
    var $_attributesHidden;

    /**
     * Default Element template string
     *
     * @var        string
     * @access     private
     */
    var $_elementTemplate = '
{javascript}
<table{class}>
<!-- BEGIN label_2 --><tr><th>{label_2}</th><!-- END label_2 -->
<!-- BEGIN label_3 --><th>&nbsp;</th><th>{label_3}</th></tr><!-- END label_3 -->
<tr>
  <td align="center">{unselected}</td>
  <td align="center">{add}{hidden}{remove}</td>
  <td align="center">{selected}</td>
</tr>
</table>';

    /**
     * Class constructor
     *
     * @param      string    Dual Select name attribute
     * @param      mixed     Label(s) for the select boxes
     * @param      mixed     Data to be used to populate options
     * @param      mixed     Either a typical HTML attribute string or an associative array
     * @access     public
     * @return     void
     */
    function HTML_QuickForm_advmultiselect($elementName=null, $elementLabel=null, $options=null, $attributes=null)
    {
        $this->HTML_QuickForm_select($elementName, $elementLabel, $options, $attributes);

        if (is_null($this->getAttribute('multiple'))) {
            // add multiple selection attribute by default if missing
            $this->updateAttributes(array('multiple' => 'multiple'));
        }
        if (is_null($this->getAttribute('size'))) {
            // default size is ten item on each select box (left and right)
            $this->updateAttributes(array('size' => 10));
        }
        if (is_null($this->getAttribute('style'))) {
            // default width of each select box
            $this->updateAttributes(array('style' => 'width:100px;'));
        }
        $this->_tableAttributes = $this->getAttribute('class');
        if (is_null($this->_tableAttributes)) {
            // default table layout
            $attr = array('border' => '0', 'cellpadding' => '10', 'cellspacing' => '0');
        } else {
            $attr = array('class' => $this->_tableAttributes);
            $this->_removeAttr('class', $this->_attributes);
        }
        $this->_tableAttributes = $this->_getAttrString($attr);

        // set default add button attributes
        $this->setButtonAttributes('add');
        // set default remove button attributes
        $this->setButtonAttributes('remove');
        // defines javascript functions names
        $this->setJsElement();
    }

    /**
     * @todo   ??? features to include there or in parent element qf_select
     * @access     public
     */
    function load(&$options, $param1=null, $param2=null, $param3=null, $param4=null)
    {
        if (is_array($options)) {

            if (isset($param1)) {
                $this->setSelected($param1);
            }
            foreach ($options as $key => $val) {
                if (isset($param2)) {
                     $attrib = @$param2[$key];
                }
                $this->addOption($val, $key, $attrib);
            }
        } else {
            parent::load($options, $param1=null, $param2=null, $param3=null, $param4=null);
        }
    }

    /**
     * Sets the button Add|Remove attributes
     *
     * @param      string    Button identifier, either 'add' or 'remove'
     * @param      mixed     (optional) Either a typical HTML attribute string or an associative array
     * @access     public
     */
    function setButtonAttributes($button, $attributes = null)
    {
        if (!is_string($button)) {
            return PEAR::raiseError('Argument 1 of advmultiselect::setButtonAttributes is not a string');
        }

        switch ($button) {
            case 'add':
                if (is_null($attributes)) {
                    $this->_addButtonAttributes = array('value' => ' >> ', 'type' => 'button');
                } else {
                    $this->_updateAttrArray($this->_addButtonAttributes, $this->_parseAttributes($attributes));
                }
                break;
            case 'remove':
                if (is_null($attributes)) {
                    $this->_removeButtonAttributes = array('value' => ' << ', 'type' => 'button');
                } else {
                    $this->_updateAttrArray($this->_removeButtonAttributes, $this->_parseAttributes($attributes));
                }
                break;
            default;
                return PEAR::raiseError('Argument 1 of advmultiselect::setButtonAttributes has unexpected value');
        }
    }

    /**
     * Sets element template
     *
     * @param      string    The HTML surrounding select boxes and buttons
     * @access     public
     * @return     void
     */
    function setElementTemplate($html)
    {
        $this->_elementTemplate = $html;
    }

    /**
     * Sets JavaScript function name parts. Maybe usefull to avoid conflict names
     *
     * @param      string    (optional) Prefix name
     * @param      string    (optional) Postfix name
     * @access     public
     * @return     void
     */
    function setJsElement($pref = null, $post = 'moveSelections')
    {
        $this->_jsPrefix  = $pref;
        $this->_jsPostfix = $post;
    }

    /**
     * Returns the HTML generated for the advanced mutliple select component
     *
     * @access     public
     * @return     string
     */
    function toHtml()
    {
        if ($this->_flagFrozen) {
            return $this->getFrozenHtml();
        } else {
            $tabs    = $this->_getTabs();
            $tab     = $this->_getTab();
            $strHtml = '';

            if ($this->getComment() != '') {
                $strHtml .= $tabs . '<!-- ' . $this->getComment() . " //-->\n";
            }

            $selectName = $this->getName() . '[]';

            // set name of Select From Box
            $this->_attributesUnselected = array('name' => '__'.$selectName, 'onDblClick' => "{$this->_jsPrefix}{$this->_jsPostfix}(this.form.elements['__" . $selectName . "'], this.form.elements['_" . $selectName . "'], this.form.elements['" . $selectName . "'], 'add')");
            $this->_attributesUnselected = array_merge($this->_attributes, $this->_attributesUnselected);
            $attrUnselected = $this->_getAttrString($this->_attributesUnselected);

            // set name of Select To Box
            $this->_attributesSelected = array('name' => '_'.$selectName, 'onDblClick' => "{$this->_jsPrefix}{$this->_jsPostfix}(this.form.elements['__" . $selectName . "'], this.form.elements['_" . $selectName . "'], this.form.elements['" . $selectName . "'], 'remove')");
            $this->_attributesSelected = array_merge($this->_attributes, $this->_attributesSelected);
            $attrSelected = $this->_getAttrString($this->_attributesSelected);

            // set name of Select hidden Box
            $this->_attributesHidden = array('name' => $selectName, 'style' => 'overflow: hidden; visibility: hidden; display: none; width: 0; height: 0; disabled: disabled');
            $this->_attributesHidden = array_merge($this->_attributes, $this->_attributesHidden);
            $attrHidden = $this->_getAttrString($this->_attributesHidden);

            // The 'unselected' multi-select which appears on the left
            $strHtmlUnselected = "<select $attrUnselected>\n";
            // The 'selected' multi-select which appears on the right
            $strHtmlSelected = "<select $attrSelected>\n";
            // The 'hidden' multi-select
            $strHtmlHidden = "<select $attrHidden>\n";

            foreach ($this->_options as $option) {
                if (is_array($this->_values) && in_array((string)$option['attr']['value'], $this->_values)) {
                    // The items is *selected* so we want to put it in the 'selected' multi-select
                    $strHtmlSelected .= $tabs . $tab ."<option" . $this->_getAttrString($option['attr']) . '>' .
                            $option['text'] . "</option>\n";
                    // Add it to the 'hidden' multi-select and set it as 'selected'
                    $strHtmlHidden .= $tabs . $tab . "<option" . $this->_getAttrString($option['attr']) . ' selected>' .
                            $option['text'] . "</option>\n";
                }
                else {
                    // The item is *unselected* so we want to put it in the 'unselected' multi-select
                    $html = $tabs . $tab . "<option" . $this->_getAttrString($option['attr']) . '>' .
                            $option['text'] . "</option>\n";
                    $strHtmlUnselected .= $html;
                    // Add it to the hidden multi-select as 'unselected'
                    $strHtmlHidden .= $html;
                }
            }
            $strHtmlSelected   .= '</select>';
            $strHtmlUnselected .= '</select>';
            $strHtmlHidden     .= '</select>';

            // build the remove button with all its attributes
            $attributes = array('onClick' => "{$this->_jsPrefix}{$this->_jsPostfix}(this.form.elements['__" . $selectName . "'], this.form.elements['_" . $selectName . "'], this.form.elements['" . $selectName . "'], 'remove'); return false;");
            $this->_removeButtonAttributes = array_merge($this->_removeButtonAttributes, $attributes);
            $attrStrRemove = $this->_getAttrString($this->_removeButtonAttributes);
            $strHtmlRemove = "<input $attrStrRemove />\n";

            // build the add button with all its attributes
            $attributes = array('onClick' => "{$this->_jsPrefix}{$this->_jsPostfix}(this.form.elements['__" . $selectName . "'], this.form.elements['_" . $selectName . "'], this.form.elements['" . $selectName . "'], 'add'); return false;");
            $this->_addButtonAttributes = array_merge($this->_addButtonAttributes, $attributes);
            $attrStrAdd = $this->_getAttrString($this->_addButtonAttributes);
            $strHtmlAdd = "<input $attrStrAdd />\n";

            // render all part of the multi select component with the template
            $strHtml = $this->_elementTemplate;

            // Prepare multiple labels
            $labels = $this->getLabel();
            if (is_array($labels)) {
                array_shift($labels);
            }
            // render extra labels, if any
            if (is_array($labels)) {
                foreach($labels as $key => $text) {
                    $key  = is_int($key)? $key + 2: $key;
                    $strHtml = str_replace("{label_{$key}}", $text, $strHtml);
                    $strHtml = str_replace("<!-- BEGIN label_{$key} -->", '', $strHtml);
                    $strHtml = str_replace("<!-- END label_{$key} -->", '', $strHtml);
                }
            }
            // clean up useless label tags
            if (strpos($strHtml, '{label_')) {
                $strHtml = preg_replace('/\s*<!-- BEGIN label_(\S+) -->.*<!-- END label_\1 -->\s*/i', '', $strHtml);
            }

            $placeHolders = array(
                '{javascript}', '{class}',
                '{unselected}', '{hidden}', '{selected}',
                '{add}', '{remove}'
            );
            $htmlElements = array(
                $this->_getJs(), $this->_tableAttributes,
                $strHtmlUnselected, $strHtmlHidden, $strHtmlSelected,
                $strHtmlAdd, $strHtmlRemove
            );

            $strHtml = str_replace($placeHolders, $htmlElements, $strHtml);

            return $strHtml;
        }
    }

    /**
     * Returns the javascript code generated to handle this element
     *
     * @access     private
     * @return     string
     */
    function _getJs()
    {
        $js = '';
        $jsfuncName = $this->_jsPrefix . $this->_jsPostfix;
        if (!defined('HTML_QUICKFORM_ADVMULTISELECT_'.$jsfuncName.'_EXISTS')) {
             // We only want to include the javascript code once per form
            define('HTML_QUICKFORM_ADVMULTISELECT_'.$jsfuncName.'_EXISTS', true);

            $js .= "
                /* begin javascript for HTML_QuickForm_advmultiselect */
                function {$jsfuncName}(menuLeft, menuRight, menuHidden, action) {
                    if(action == 'add') {
                        menuFrom = menuLeft;
                        menuTo = menuRight;
                    }
                    else {
                        menuFrom = menuRight;
                        menuTo = menuLeft;
                    }
                    // Don't do anything if nothing selected. Otherwise we throw jscript errors.
                    if(menuFrom.selectedIndex == -1) {
                        return;
                    }

                    // Add items to the 'TO' list.
                    for ( i=0; i < menuFrom.length; i++) {
                        if (menuFrom.options[i].selected == true ) {
                            menuTo.options[menuTo.length]= new Option(menuFrom.options[i].text, menuFrom.options[i].value);
                        }
                    }

                    // Remove items from the 'FROM' list.
                    for ( i = (menuFrom.length - 1); i>=0; i--){
                        if (menuFrom.options[i].selected == true ) {
                            menuFrom.options[i] = null;
                        }
                    }

                    // Unselect all options in the hidden select.
                    for ( i=0; i < menuHidden.length; i++) {
                        menuHidden.options[i].selected = false;
                    }

                    // Set the appropriate items as 'selected in the hidden select.
                    // These are the values that will actually be posted with the form.
                    for ( i=0; i < menuRight.length; i++) {
                        menuHidden.options[menuHidden.length] = new Option(menuRight.options[i].text, menuRight.options[i].value);
                        menuHidden.options[menuHidden.length-1].selected = true;
                    }
                }
                /* end javascript for HTML_QuickForm_advmultiselect */
            ";

            $js = "<script type=\"text/javascript\">\n//<![CDATA[\n" . $js . "//]]>\n</script>";
        }
        return $js;
    }
}

if (class_exists('HTML_QuickForm')) {
    HTML_QuickForm::registerElementType('advmultiselect', 'HTML/QuickForm/advmultiselect.php', 'HTML_QuickForm_advmultiselect');
}
?>